if (!window.onerror) {
    window.onerror = function(msg, url, line) {
        logError(msg + "[" + url + "]" + line);
    }
}

function logInfo(msg) {
    console.log("VKAppBridge: [INFO] " + msg);
}

function logError(msg) {
    console.log("VKAppBridge: [ERROR] " + msg);
}

function logWarn(msg) {
    console.log("VKAppBridge: [WARNING] " + msg);
}

window.VKAppWebViewBridge = {
    registerHandler: registerHandler,
    callHandler: callHandler,
    disableJavscriptAlertBoxSafetyTimeout: disableJavscriptAlertBoxSafetyTimeout,
    _fetchQueue: _fetchQueue,
    _receiveFromNative: _receiveFromNative
};
window.vkappbridge = {
    receiveFromNative: _receiveFromNative
};

var OS_TYPE = 'android';

function _pid() {
    var num = '';
    for (var i = 0; i < 6; i++) {
        num += Math.floor(Math.random() * 10); 
    }
    return num;
}

var sPid = _pid();
var sBaseCount = 0;
var sOnlineCount = 0;
var sThirdCount = 0;
var sCount = 0;

var REQUEST_TYPE_OP = 'op';
var REQUEST_TYPE_OD = 'od';
var REQUEST_TYPE_DAP = 'dap';

var sMessageIframe;
var sMessageQueueForSend = [];
var sMessageHandlers = {};

var MESSAGE_SCHEME_PROTOCAL = 'https';
var MESSAGE_SCHEME_BRIDGE = 'vkappbridge';
var MESSAGE_SCHEME_NAMESPACE_NET = 'net';
var MESSAGE_SCHEME_HOST = '__vkappbridge_message_queue__';

var sResponseCallbacks = {};
var sUniqueId = 1;
var sDispatchMessagesWithTimeoutSafety = true;

var STYPE_BASE = 'base';
var STYPE_THIRD = 'third';
var STYPE_ONLINE = 'online';

var SNAME_BASE = '基础服务';
var SNAME_THIRD = '第三方服务';
var SNAME_ONLINE = '系统服务';

function _createRequest(id, name, host, type) {
    return {id: id, name: name, host: host, type: type};
}

var sBaseRequestArray = new Array();
sBaseRequestArray[0] = _createRequest(1, '基本数据', 'rpc.vipkid.com.cn', REQUEST_TYPE_DAP);
sBaseRequestArray[1] = _createRequest(2, '课程', 'parent-rest.vipkid.com.cn', REQUEST_TYPE_DAP);
sBaseRequestArray[2] = _createRequest(3, '课程2', 'parent-app.vipkid.com.cn', REQUEST_TYPE_DAP);
sBaseRequestArray[3] = _createRequest(4, '约课', 'mobile.vipkid.com.cn', REQUEST_TYPE_DAP);
sBaseRequestArray[4] = _createRequest(5, '回放验证', 'api.duobeiyun.com', REQUEST_TYPE_DAP);
sBaseRequestArray[5] = _createRequest(6, '回放连接', 'playback2.duobeiyun.com', REQUEST_TYPE_DAP);
sBaseRequestArray[6] = _createRequest(7, '回放连接2', 'playback3.duobeiyun.com', REQUEST_TYPE_DAP);
sBaseRequestArray[7] = _createRequest(8, '监课线路', 'ms.vipkid.com.cn', REQUEST_TYPE_DAP);

var sOnlineRequestArray = new Array();

var sThirdRequestArray = new Array();
sThirdRequestArray[0] = _createRequest(1, '百度', 'www.baidu.com', REQUEST_TYPE_DAP);
sThirdRequestArray[1] = _createRequest(2, '微信平台', 'mp.weixin.qq.com', REQUEST_TYPE_DAP);

var sLoadOnlineScriptFinishOrTimeout = false;

function _isIOS() {
    return OS_TYPE == 'ios';
}

function _isAndroid() {
    return OS_TYPE == 'android';
}

function registerHandler(handlerName, handler) {
    sMessageHandlers[handlerName] = handler;
}

function callHandler(handlerName, data, responseCallback) {
    if (arguments.length == 2 && typeof data == 'function') {
        responseCallback = data;
        data = null;
    }
    if (_isIOS()) {
        _doSend({handlerName: handlerName, data: data}, responseCallback);
    }
    if (_isAndroid()) {
        _doAndroidSend({handlerName: handlerName, data: data}, responseCallback);
    }
}

function disableJavscriptAlertBoxSafetyTimeout() {
    sDispatchMessagesWithTimeoutSafety = false;
}

function _doAndroidSend(message, responseCallback) {
    var callbackId = '';
    if (responseCallback) {
        callbackId = '_vkappbridge_' + (sUniqueId++) + '_' + new Date().getTime();
        sResponseCallbacks[callbackId] = responseCallback;
    }
    var params;
    if (message.data && message.data != '') {
        params = JSON.stringify(message.data);
    } else {
        params = JSON.stringify({});
    }
    var callback;
    if (callbackId == '') {
        callback = '';
    } else {
        callback = '#' + callbackId;
    }
    var command = MESSAGE_SCHEME_PROTOCAL + '://' + MESSAGE_SCHEME_BRIDGE + '/' + MESSAGE_SCHEME_NAMESPACE_NET + '/' + message.handlerName + '?' + encodeURIComponent(params) + callback;
    var result = window.prompt(command);
}

function _doSend(message, responseCallback) {
    if (responseCallback) {
        var callbackId = '_vkappbridge_' + (sUniqueId++) + '_' + new Date().getTime();
        sResponseCallbacks[callbackId] = responseCallback;
        message['callbackId'] = callbackId;
    }
    sMessageQueueForSend.push(message);
    sMessageIframe.src = MESSAGE_SCHEME_PROTOCAL + "://" + MESSAGE_SCHEME_HOST;
}

function _fetchQueue() {
    var messageQueueString = JSON.stringify(sMessageQueueForSend);
    sMessageQueueForSend = [];
    return messageQueueString;
}

function _dispatchMessageFromNative(messageJSON) {
    if (sDispatchMessagesWithTimeoutSafety) {
        setTimeout(_doDispatchMessageFromNative);
    } else {
        _doDispatchMessageFromNative();
    }

    function _doDispatchMessageFromNative() {
        var message = JSON.parse(messageJSON);
        var messageHandler;
        var responseCallback;

        if (message.responseId) {
            responseCallback = sResponseCallbacks[message.responseId];
            if (!responseCallback) {
                return;
            }
            responseCallback(message.responseData);
            delete sResponseCallbacks[message.responseId];
        } else {
            if (message.callbackId) {
                var callbackResponseId = message.callbackId;
                responseCallback = function(responseData) {
                    _doSend({handlerName: message.handlerName, responseId: callbackResponseId, responseData: responseData});
                };
            }
            var handler = sMessageHandlers[message.handlerName];
            if (!handler) {
                logError("no handler for message from ObjC: " + message);
            } else {
                handler(message.data, responseCallback);
            }
        }
    }
}

function _receiveFromNative(messageJSON) {
    _dispatchMessageFromNative(messageJSON);
}

function _updateSpace(space) {
    if (sBaseCount <= 0 && sThirdCount <= 0 && sOnlineCount <= 0) {
        logInfo('inner:' + sBaseCount + ',' + sThirdCount + ',' + sOnlineCount);
        _vupdateInfo('本次检测编号: ' + sPid);
        _vupdateState('检测完成');
    } else {
        sCount = sCount + 1;
        if (sCount >= 99) {
            sCount = 99;
        }
        _vupdateInfo('' + sCount + '%');
    }
    
    if (space == STYPE_BASE) {
        if (sBaseCount > 0) {
            _vshowBaseLoading();
        } else {
            _vhideBaseLoading();
        }
    } else if (space == STYPE_THIRD) {
        if (sThirdCount > 0) {
            _vshowThirdLoading();
        } else {
            _vhideThirdLoading();
        }
    } else if (space == STYPE_ONLINE) {
        if (sOnlineCount > 0) {
            _vshowOnlineLoading();
        } else {
            _vhideOnlineLoading();
        }
    }
}

function _updateContent(space, id, response) {
    _updateSpace(space);
    
    var requestArray;
    if (space == STYPE_BASE) {
        requestArray = sBaseRequestArray;
    } else if (space == STYPE_THIRD) {
        requestArray = sThirdRequestArray;
    } else if (space == STYPE_ONLINE) {
        requestArray = sOnlineRequestArray;
    }
    if (requestArray) {
        _updateContentById(space, id, requestArray, response);
    }
}

function _autoCount() {
    if (sBaseCount <= 0 && sThirdCount <= 0 && sOnlineCount <= 0) {
        return;
    }
    sCount = sCount + 1;
    if (sCount >= 99) {
        sCount = 99;
    }
    _vupdateInfo('' + sCount + '%');
    setTimeout(_autoCount, 100);
}

function _updateContentById(space, id, array, response) {
    var request;
    for (var i = 0; i < array.length; i++) {
        if (array[i].id == id) {
            request = array[i].name;
        }
    }
    if (!request) {
        return;
    }
    if (space == STYPE_BASE) {
        _vupdateBaseContent(id, request, response);
    } else if (space == STYPE_THIRD) {
        _vupdateThirdContent(id, request, response);
    } else if (space == STYPE_ONLINE) {
        _vupdateOnlineContent(id, request, response);
    }
}

function _increaseCount(space) {
    if (space == STYPE_BASE) {
        sBaseCount = sBaseCount + 1;
    } else if (space == STYPE_THIRD) {
        sThirdCount = sThirdCount + 1;
    } else if (space == STYPE_ONLINE) {
        sOnlineCount = sOnlineCount + 1;
    }
}

function _decreaseCount(space) {
    if (space == STYPE_BASE) {
        sBaseCount = sBaseCount - 1;
    } else if (space == STYPE_THIRD) {
        sThirdCount = sThirdCount - 1;
    } else if (space == STYPE_ONLINE) {
        sOnlineCount = sOnlineCount - 1;
    }
}

function _init() {
    if (_isIOS()) {
        sMessageIframe = document.createElement('iframe');
        sMessageIframe.style.display = 'none';
        sMessageIframe.src = MESSAGE_SCHEME_PROTOCAL + '://' + MESSAGE_SCHEME_HOST;
        document.documentElement.appendChild(sMessageIframe);
    }
    if (_isAndroid()) {
        // empty
    }
}

function _checkDnsAndPing(space, id, host) {
    _increaseCount(space);
    _updateContent(space, id, '');
    callHandler('dns', {pid: sPid, host: host, timeout: 5, server: ''}, function(response) {
        _decreaseCount(space);
        if (response && response.pid == sPid) {
            if (!response.ipList) {
                _checkPing(space, id, host);
                return;
            }
            var length = response.ipList.length;
            if (length == 0) {
                _checkPing(space, id, host);
                return;
            }
            for (var i = 0; i < length; i++) {
                var ip = response.ipList[i];
                if (ip && ip.length > 0) {
                    _checkPing(space, id, ip);
                }
            }
            return;
        }
        _updateContent(space, id, '检测失败');
    });
}

function _checkPing(space, id, dest) {
    _increaseCount(space);
    _updateContent(space, id, '');
    callHandler('ping', {pid: sPid, dest: dest, timeout: 5, queryCount: 5, interval: 1, packageSize: 64}, function(response) {
        _decreaseCount(space);
        if (response && response.pid == sPid) {
            var result;
            if (response.minRtt && response.maxRtt && response.minRtt > 0 && response.maxRtt > 0) {
                result = '' + response.minRtt + '~' + response.maxRtt + 'ms' + '(' + response.lossRate + ')';
            } else {
                result = '解析失败';
            }
            _updateContent(space, id, result);
            return;
        }
        _updateContent(space, id, '检测失败');
    });
}

function _checkDns(space, id, host) {
    _increaseCount(space);
    _updateContent(space, id, '');
    callHandler('dns', {pid: sPid, host: host, timeout: 5, server: ''}, function(response) {
        _decreaseCount(space);
        if (response && response.pid == sPid) {
            var result;
            if (response.ipList && response.ipList.length > 0) {
                result = '解析成功';
            } else {
                result = '解析失败';
            }
            _updateContent(space, id, result);
            return;
        }
        _updateContent(space, id, '检测失败');
    });
}

function checkBase() {
    for (var i = 0; i < sBaseRequestArray.length; i++) {
        var request = sBaseRequestArray[i];
        _check(STYPE_BASE, request);
    }
    if (sBaseRequestArray.length == 0) {
        _updateSpace(STYPE_BASE);
    }
}

function checkThird() {
    for (var i = 0; i < sThirdRequestArray.length; i++) {
        var request = sThirdRequestArray[i];
        _check(STYPE_THIRD, request);
    }
    if (sThirdRequestArray.length == 0) {
        _updateSpace(STYPE_THIRD);
    }
}

function _checkOnline() {
    for (var i = 0; i < sOnlineRequestArray.length; i++) {
        var request = sOnlineRequestArray[i];
        _check(STYPE_ONLINE, request);
    }
    if (sOnlineRequestArray.length == 0) {
        _updateSpace(STYPE_ONLINE);
    }
}

function checkOnline() {
    _increaseCount(STYPE_ONLINE);
    _loadScript('https://resource.vipkid.com.cn/parent-app/config/dynamic.js', function() {
        if (sLoadOnlineScriptFinishOrTimeout) {
            return;
        }
        sLoadOnlineScriptFinishOrTimeout = true;
        _decreaseCount(STYPE_ONLINE);
        try {
            dynamicLoadOnlineArray(sOnlineRequestArray);
        } catch (e) {
            // empty
        } finally {
            // empty
        }
        _checkOnline();
    });
    setTimeout(function() {
        if (sLoadOnlineScriptFinishOrTimeout) {
            return;
        }
        sLoadOnlineScriptFinishOrTimeout = true;
        _decreaseCount(STYPE_ONLINE);
        _checkOnline();
    }, 10000);
    _updateSpace(STYPE_ONLINE);
}

function _check(space, request) {
    if (request) {
        if (request.type == REQUEST_TYPE_OD) {
            _checkDns(space, request.id, request.host);
        } else if (request.type == REQUEST_TYPE_OP) {
            _checkPing(space, request.id, request.host);
        } else if (request.type == REQUEST_TYPE_DAP) {
            _checkDnsAndPing(space, request.id, request.host);
        }
    }
}

function _loadScript(url, callback) {
    var script = document.createElement('script');
    script.type = 'text/javascript';
    script.onload = function() {
        callback();
    };
    script.src = url;
    document.body.appendChild(script);
}
